#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import os
import tempfile
import shutil
import fileinput
import sys


def write_patch(f):
    f.write('\tcard_atr 3b:ff:18:00:ff:81:31:fe:55:00:6b:02:09:03:03:01:11:01:43:4e:53:11:31:80:8c {\n')
    f.write('\t\tname = " T&S DS/2048 (disabled)";\n')
    f.write('\t\tdriver = "westcos";\n')
    f.write('\t}\n')


def disable_bit4id_move(conf):
    """
    Add section to disable Bit4id cards in OpenSC config file using temporary file
    :param conf: OpenSC config file
    """
    path = None
    with open(conf, 'r') as inf, tempfile.NamedTemporaryFile(delete=False) as o:
        path = o.name
        for line in inf:
            o.write(line)
            if "app default {" in line:
                write_patch(o)
    if path is not None:
        shutil.move(path, conf)


def disable_bit4id_inplace(conf):
    """
    Add section to disable Bit4id cards in OpenSC config file inplace
    :param conf: OpenSC config file
    """
    for line in fileinput.input(conf, inplace=1):
        sys.stdout.write(line)
        if "app default {" in line:
            write_patch(sys.stdout)


def disabled(conf):
    """
    Parser for OpenSC config file, detect if Bit4id cards are disabled.
    :param conf: path to OpenSC configuration file
    :return: True if Bit4id cards are disabled or False otherwise
    """
    atr = '3b:ff:18:00:ff:81:31:fe:55:00:6b:02:09:03:03:01:11:01:43:4e:53:11:31:80:8c'
    regexp = 'card_atr\s+' + atr + '\s*\{\s*name\s*=\s*"[\w\W]+";\s*driver\s*=\s*"[\w\W]+";\s*\}'
    r = re.compile(regexp)
    comment = re.compile('^\s*#')
    clean_conf = ''
    with open(conf) as cfg:
        for line in cfg:
            if comment.match(line):
                continue
            clean_conf += line
        return re.search(regexp, clean_conf) is not None


config_path = "/Library/OpenSC/etc/opensc.conf"
try:
    if not os.path.isfile(config_path):
        sys.exit(2)  # Config file is missing
    if not disabled(config_path):
        disable_bit4id_inplace(config_path)
        if disabled(config_path):
            sys.exit(0)  # Success
        sys.exit(3)  # Operation failed
    else:
        sys.exit(1)  # Already disabled
except Exception:
    sys.exit(4)  # Exception is thrown